<?php

namespace Drupal\openai_client\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\openai_client\OpenAIClientWrapper;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Render\Markup;

/**
 * OpenAI Assistant chat form.
 */
class AssistantChatForm extends FormBase {

  /**
   * Wrapper for the openai client.
   *
   * @var \Drupal\openai_client\OpenAIClientWrapper
   */
  protected OpenAIClientWrapper $api;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  public function __construct(OpenAIClientWrapper $api, RequestStack $requestStack) {
    $this->api = $api;
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): AssistantChatForm {
    return new static(
      $container->get('openai_client.wrapper'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openai_client_assistant_chat_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $assistant_id = NULL): array {
    $session = $this->requestStack->getCurrentRequest()->getSession();
    $conversation = $session->get('openai_client_chat_' . $assistant_id, []);

    $form['#prefix'] = '<div id="assistant-chat-wrapper">';
    $form['#suffix'] = '</div>';

    $form['conversation'] = [
      '#type' => 'markup',
      '#markup' => $this->renderConversation($conversation),
    ];

    $form['message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Your Message'),
      '#required' => TRUE,
      '#attributes' => ['autocomplete' => 'off'],
    ];

    $form['assistant_id'] = [
      '#type' => 'hidden',
      '#value' => $assistant_id,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['send'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#ajax' => [
        'callback' => '::ajaxSubmit',
        'wrapper' => 'assistant-chat-wrapper',
        'effect' => 'fade',
      ],
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Normal submit not used, AJAX only.
  }

  /**
   * Ajax callback for form submission.
   *
   * Ignore submitForm(); this is the real submit function.
   *
   * @param array $form
   *   Form render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax response.
   */
  public function ajaxSubmit(array &$form, FormStateInterface $form_state): AjaxResponse {
    $assistant_id = $form_state->getValue('assistant_id');
    $user_message = $form_state->getValue('message');
    $session = $this->requestStack->getCurrentRequest()->getSession();
    $conversation_key = 'openai_client_chat_' . $assistant_id;
    $conversation = $session->get($conversation_key, []);
    $conversation[] = ['role' => 'user', 'content' => $user_message];

    // Call OpenAI API.
    try {
      // Step 1: Create conversation thread.
      $thread = $this->api->getClient()->threads()->create([
        'messages' => $conversation,
      ]);

      // Step 2: Add user message to thread.
      $parameters = [
        'role' => 'user',
        'content' => $user_message,
      ];
      $this->api->getClient()->threads()->messages()->create($thread->id, $parameters);

      // Step 3: Create a Run.
      $runParameters = [
        'assistant_id' => $assistant_id,
      ];
      $run = $this->api->getClient()->threads()->runs()->create($thread->id, $runParameters);

      // Step 4: Get assistant response.
      $run = $this->api->waitForRunCompletion($thread->id, $run->id);
      if ($run->status === 'completed') {
        $messages = $this->api->getClient()->threads()->messages()->list($thread->id, ['run_id' => $run->id]);
        $assistant_message = '';
        foreach (array_reverse($messages->data) as $msg) {
          if ($msg->role === 'assistant') {
            // Try to extract the assistant's message content.
            if (!empty($msg->content) && isset($msg->content[0]->text->value)) {
              $assistant_message = $msg->content[0]->text->value;
            }
            elseif (!empty($msg->content) && isset($msg->content[0]->value)) {
              $assistant_message = $msg->content[0]->value;
            }
            break;
          }
        }
        $conversation[] = ['role' => 'assistant', 'content' => $assistant_message];
      }
      else {
        $conversation[] = ['role' => 'assistant', 'content' => $this->t('Error: Sorry, run did not complete.')];
      }
    }
    catch (\Exception $e) {
      $conversation[] = ['role' => 'assistant', 'content' => $this->t('Error: @msg', ['@msg' => $e->getMessage()])];
    }

    $session->set($conversation_key, $conversation);
    $form['conversation']['#markup'] = $this->renderConversation($conversation);
    $form_state->setValue('message', '');
    $form['message']['#value'] = '';

    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('#assistant-chat-wrapper', \Drupal::service('renderer')->render($form)));
    return $response;
  }

  /**
   * Render conversation.
   *
   * Includes logic to handle both user and assistant messages.
   *
   * @param array $conversation
   *   Conversation array.
   *
   * @return string
   *   Rendered conversation.
   */
  protected function renderConversation(array $conversation): string {
    $output = '';
    foreach ($conversation as $entry) {
      $role = $entry['role'] === 'user' ? '<strong>You:</strong>' : '<strong>Assistant:</strong>';

      // Response might be a string, or it might be TranslatableMarkup.
      // Either way we can cast it to a string.
      $response = (string) $entry['content'];
      $output .= '<div class="chat-msg chat-msg-' . $entry['role'] . '">' . $role . ' ' . Markup::create(nl2br(htmlspecialchars($response))) . '</div>';
    }
    return $output ?: '<em>' . $this->t('No messages yet.') . '</em>';
  }

}
