<?php

namespace Drupal\openai_client\Form;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\openai_client\OpenAIClientWrapper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a OpenAI Client form.
 */
class CreateImageForm extends FormBase {

  /**
   * OpenAI client.
   *
   * @var \Drupal\openai_client\OpenAIClientWrapper
   */
  protected OpenAIClientWrapper $api;

  /**
   * Constructs a new CreateImageForm object.
   *
   * @param \Drupal\openai_client\OpenAIClientWrapper $api
   *   OpenAI client API.
   */
  public function __construct(OpenAIClientWrapper $api) {
    $this->api = $api;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): CreateImageForm {
    return new static(
      $container->get('openai_client.wrapper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'openai_client_create_image';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $rebuildInfo = $form_state->getRebuildInfo();
    if (isset($rebuildInfo['images'])) {
      foreach ($rebuildInfo['images'] as $key => $image) {
        $form[$key] = [
          '#type' => 'markup',
          '#markup' => '<img src="' . $image . '"/>',
        ];
      }
    }

    $form['prompt'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Create image prompt'),
      '#required' => TRUE,
    ];

    $form['amount'] = [
      '#type' => 'number',
      '#title' => $this->t('Amount'),
      '#default_value' => $rebuildInfo['amount'] ?? 1,
      '#required' => TRUE,
      '#min' => 1,
      '#max' => 1,
      '#attributes' => ['readonly' => 'readonly'],
    ];

    $form['size'] = [
      '#type' => 'select',
      '#title' => $this->t('Image size'),
      '#required' => TRUE,
      '#default_value' => $rebuildInfo['size'] ?? OpenAIClientWrapper::IMAGE_SIZE_1024,
      '#options' => OpenAIClientWrapper::getImageSizes(),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $prompt = $form_state->getValue('prompt');
    $amount = $form_state->getValue('amount');
    $size = $form_state->getValue('size');
    $model = $this->api->getDefaultModel();

    $images = $this->api->imageCreate($prompt, $amount, $size, $model);
    if (isset($images["data"])) {
      $imageUrls = [];
      foreach ($images["data"] as $image) {
        // Different handling for gpt-image-1.
        if ($model === 'gpt-image-1') {
          $encodedImage = $image['b64_json'];
          $decoded = base64_decode($encodedImage);

          // Specify the file path in a subdirectory under public://.
          $directory = 'public://openai_images';
          \Drupal::service('file_system')->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
          $filename = $directory . '/generated_image_' . time() . '_' . uniqid() . '.png';

          // Save the file directly using writeData.
          $file = \Drupal::service('file.repository')->writeData($decoded, $filename);

          // Check if the file was created successfully.
          if ($file) {
            $imageUrls[] = \Drupal::service('file_url_generator')->generateAbsoluteString($file->getFileUri());
          }
          else {
            $this->messenger()->addError('Failed to save the image file.');
          }
        }
        else {
          $imageUrls[] = $image['url'];
        }
      }
      $form_state->setRebuildInfo(['images' => $imageUrls]);
      $this->messenger()->addStatus($this->t('The image/s was generated.'));
    }
    else {
      $this->messenger()->addWarning($this->t('The image/s was not generated.'));
    }

    $form_state->setRebuild();
  }

}
